import "reflect-metadata";
export declare const DecoratorOption: unique symbol;
export declare const DecoratorId: unique symbol;
declare type Class = new (...arg: any[]) => any;
export declare type DecoratorTargetType = "Method" | "Class" | "Parameter" | "Property" | "Constructor";
export interface Decorator {
    targetType: DecoratorTargetType;
    target: string;
    value: any;
    inherit: boolean;
    allowMultiple: boolean;
}
export interface ParamDecorator extends Decorator {
    targetType: "Parameter";
    targetIndex: number;
}
export declare type Reflection = ParameterReflection | FunctionReflection | PropertyReflection | MethodReflection | ClassReflection | ObjectReflection;
export interface ReflectionBase {
    kind: string;
    name: string;
}
export interface ParameterReflection extends ReflectionBase {
    kind: "Parameter";
    properties: string | {
        [key: string]: string[];
    };
    decorators: any[];
    type?: any;
    typeClassification?: "Class" | "Array" | "Primitive";
}
export interface PropertyReflection extends ReflectionBase {
    kind: "Property";
    decorators: any[];
    type?: any;
    get: any;
    set: any;
    typeClassification?: "Class" | "Array" | "Primitive";
}
export interface MethodReflection extends ReflectionBase {
    kind: "Method";
    parameters: ParameterReflection[];
    returnType: any;
    decorators: any[];
    typeClassification?: "Class" | "Array" | "Primitive";
}
export interface ConstructorReflection extends ReflectionBase {
    kind: "Constructor";
    parameters: ParameterReflection[];
}
export interface FunctionReflection extends ReflectionBase {
    kind: "Function";
    parameters: ParameterReflection[];
    returnType: any;
}
export interface ClassReflection extends ReflectionBase {
    kind: "Class";
    ctor: ConstructorReflection;
    methods: MethodReflection[];
    properties: PropertyReflection[];
    decorators: any[];
    type: Class;
    typeClassification?: "Class" | "Array" | "Primitive";
}
export interface ObjectReflection extends ReflectionBase {
    kind: "Object";
    members: Reflection[];
}
export interface ArrayDecorator {
    kind: "Array";
    type: Class;
}
export interface TypeDecorator {
    kind: "Override";
    type: Class;
    info?: string;
}
export interface PrivateDecorator {
    kind: "Ignore";
}
export interface DecoratorOption {
    inherit?: boolean;
    allowMultiple?: boolean;
}
export declare type PropertyDecorator = (target: Object, propertyKey: string | symbol, ...index: any[]) => void;
export declare const DECORATOR_KEY = "plumier.key:DECORATOR";
export declare const DESIGN_TYPE = "design:type";
export declare const DESIGN_PARAMETER_TYPE = "design:paramtypes";
export declare const DESIGN_RETURN_TYPE = "design:returntype";
export declare function getParameterNames(fn: Function): (string | {
    [key: string]: string[];
})[];
export declare function getMethodParameters(fn: Class, method: string): (string | {
    [key: string]: string[];
})[];
export declare function getConstructorParameters(fn: Class): (string | {
    [key: string]: string[];
})[];
export declare function getMembers(fun: Function): string[];
export declare function useCache<K, P extends any[], R>(cache: Map<K, R>, fn: (...args: P) => R, getKey: (...args: P) => K): (...args: P) => R;
export declare function decorateParameter(callback: ((target: Class, name: string, index: number) => any), option?: DecoratorOption): ParameterDecorator;
export declare function decorateParameter(data: any, option?: DecoratorOption): ParameterDecorator;
export declare function decorateMethod(callback: ((target: Class, name: string) => any), option?: DecoratorOption): MethodDecorator;
export declare function decorateMethod(data: any, option?: DecoratorOption): MethodDecorator;
export declare function decorateProperty(callback: ((target: Class, name: string, index?: any) => any), option?: DecoratorOption): PropertyDecorator;
export declare function decorateProperty(data: any, option?: DecoratorOption): PropertyDecorator;
export declare function decorateClass(callback: ((target: Class) => any), option?: DecoratorOption): ClassDecorator;
export declare function decorateClass(data: any, option?: DecoratorOption): ClassDecorator;
export declare function decorate(data: any | ((...args: any[]) => any), targetTypes?: DecoratorTargetType[], option?: Partial<DecoratorOption>): (...args: any[]) => void;
export declare function mergeDecorator(...fn: (ClassDecorator | PropertyDecorator | ParamDecorator | MethodDecorator)[]): (...args: any[]) => void;
/**
 * Reflect module
 * @param path module name
 */
export declare function reflect(path: string): ObjectReflection;
export declare namespace reflect {
    var noop: () => (...args: any[]) => void;
    var ignore: () => (...args: any[]) => void;
    var type: (type: Class | Class[], info?: string | undefined) => (...args: any[]) => void;
    var array: (type: Class) => (...args: any[]) => void;
    var parameterProperties: () => ClassDecorator;
}
/**
 * Reflect class
 * @param classType Class
 */
export declare function reflect(classType: Class): ClassReflection;
export declare namespace reflect {
    var noop: () => (...args: any[]) => void;
    var ignore: () => (...args: any[]) => void;
    var type: (type: Class | Class[], info?: string | undefined) => (...args: any[]) => void;
    var array: (type: Class) => (...args: any[]) => void;
    var parameterProperties: () => ClassDecorator;
}
export default reflect;
